#include <cstdlib>
#include <fstream>
#include <iostream>

#include "rlcsa.h"


using namespace CSA;


/*
  This program writes the ILCP array of the collection into a file.
*/


int
main(int argc, char** argv)
{
  std::cout << "ILCP builder" << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: build_ilcp base_name" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  std::string ilcp_name = base_name + ILCP_EXTENSION;
  std::cout << "ILCP: " << ilcp_name << std::endl;
  std::ofstream ilcp_file(ilcp_name.c_str(), std::ios_base::binary);
  if(!ilcp_file)
  {
    std::cerr << "Error creating ILCP file!" << std::endl;
    return 2;
  }
  std::cout << std::endl;
  RLCSA rlcsa(base_name);

  double start = readTimer();
  uint* iLCP = new uint[rlcsa.getSize()];
  for(usint i = 0; i < rlcsa.getNumberOfSequences(); i++)
  {
    SuffixArray* sa = rlcsa.getSuffixArrayForSequence(i);
    uint* plcp = sa->getLCPArray(true);
    for(usint j = 0; j < sa->getSize(); j++)
    {
      iLCP[sa->getRanks()[j] - rlcsa.getNumberOfSequences()] = plcp[j];
    }
    delete[] plcp;
    delete sa;
  }

  double total_time = readTimer() - start;
  double megabytes = rlcsa.getSize() / (double)MEGABYTE;
  std::cout << "Processed " << megabytes << " megabytes in " << total_time << " seconds (" << (megabytes / total_time) << " MB/s)." << std::endl;
  std::cout << "Memory: " << memoryUsage() << " kB" << std::endl;
  std::cout << std::endl;

  ilcp_file.write((char*)iLCP, rlcsa.getSize() * sizeof(uint));
  delete[] iLCP;
  ilcp_file.close();

  return 0;
}
